import {
  Message
} from 'element-ui';
import {
  dataapiDetail,
  openurl
} from './report';
import {
  optionList,
  enumCheck
} from '@/api/newbi/bigdata';
import {
  dictDetail
} from '@/api/newbi/dict';
import {
  param
} from "./config";
// 整理树结构
export const countTreeOptions = (originData) => {
  originData.forEach((i) => {
    i.children = [];
    i.KEY = i.KEY || i.VALUE;
  });
  let parents = [...originData];
  let res = [],
    childrens = [];
  originData.forEach((i) => {
    if (i.PID == "0") {
      res.push(i);
    } else {
      childrens.push(i);
    }
  });
  childrens.forEach((i) => {
    let parent = parents.find((j) => j.ID == i.PID);
    if (parent) {
      parent.children.push(i);
    }
  });
  return res;
};
// 去掉子项空数组
export const clearChildren = node => {
  if (!node.children) return;
  if (!node.children.length) {
    delete node.children;
  } else {
    node.children.forEach(clearChildren);
  }
};
// 获取枚举可选值
export const getEnumData = (filter, cubeDataId, visualId, componentId, isEdit) => {
  let {
    enumSource,
    enumId
  } = filter;
  if (enumSource == "1") {
    filter.loading = true;
    let {
      tableName,
      columnName
    } = filter;
    if (!tableName || !columnName) {
      tableName = filter.column.tableName;
      columnName = filter.column.columnName;
    }
    if (isEdit) {
      enumCheck({
        tableName,
        columnName,
        cubeDataId
      }).then(() => {
        optionList({
          tableName,
          columnName,
          cubeDataId,
          visualId,
          componentId
        }).then((res) => {
          filter.options = res.data.data.map((i) => ({
            KEY: i[columnName],
            VALUE: i[columnName],
          }));
          filter.loading = false;
        });
      }).catch(() => {
        Message.warning('此数据集不支持原始数据聚合');
        filter.loading = false;
      });
    } else {
      optionList({
        tableName,
        columnName,
        cubeDataId,
        visualId,
        componentId
      }).then((res) => {
        filter.options = res.data.data.map((i) => ({
          KEY: i[columnName],
          VALUE: i[columnName],
        }));
        filter.loading = false;
      });
    }
  } else if (enumSource == "2") {
    if (!enumId) {
      return Message.warning('请选择api');
    }
    filter.loading = true;
    dataapiDetail(enumId).then((res) => {
      let {
        apiKey
      } = res.data.data;
      openurl(apiKey, param).then((res) => {
        filter.loading = false;
        let data = res.data.data;
        if (!data.length) {
          filter.options = data;
        }
        if (data[0].ID) {
          let tree = countTreeOptions(data);
          tree.forEach(clearChildren);
          filter.options = tree;
        } else {
          filter.options = data.map(i => ({
            KEY: i.KEY || i.VALUE,
            VALUE: i.VALUE
          }));
        }
      });
    });
  } else {
    if (!enumId) {
      return Message.warning('请选择字典');
    }
    dictDetail({
      id: enumId
    }).then((res) => {
      let content = JSON.parse(res.data.data.content || '[]');
      filter.options = content.map(i => ({
        KEY: i.key,
        VALUE: i.value
      }));
    });
  }
};
export const onEnumSelectAll = filter => {
  let options = filter.options;
  let arr = [];
  options.forEach((i) => getOptionValue(arr, i));
  filter.conditions[0].values = arr;
};
export const getOptionValue = (arr, item) => {
  if (item.children) {
    item.children.forEach((i) => {
      getOptionValue(arr, i);
    });
  } else {
    arr.push(item.KEY);
  }
};
